/**********************************************************************
 *
 * documentaction.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: documentaction.cpp 257 1999-05-10 03:40:44Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.2  1999/05/10 03:40:35  sjboddie
   lots of changes - slowly getting document action sorted out

   Revision 1.1  1999/04/30 01:59:40  sjboddie
   lots of stuff - getting documentaction working (documentaction replaces
   old browseaction)

   Revision 1.2  1999/03/29 02:14:27  sjboddie

   More changes to browseaction

   Revision 1.1  1999/03/25 03:09:40  sjboddie

   subjectbrowseaction became browseaction


 */


#include "documentaction.h"
#include "browsetools.h"
#include "OIDtools.h"

documentaction::documentaction () {
  // this action uses cgi variables "a", "d", "cl",
  // "x", "gc", "gt", "gp", and "hl"
  cgiarginfo arg_ainfo;
  arg_ainfo.shortname = "a";
  arg_ainfo.longname = "action";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "p";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);

  arg_ainfo.shortname = "d";
  arg_ainfo.longname = "document OID";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::none;
  arg_ainfo.argdefault = "";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  arg_ainfo.shortname = "cl";
  arg_ainfo.longname = "classification OID";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::none;
  arg_ainfo.argdefault = "";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "gc" controls the expand/contract
  // contents function
  arg_ainfo.shortname = "gc";
  arg_ainfo.longname = "expand contents";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "gt" controls the expand/contract
  // text function 0 = not expanded, 1 = expand unless 
  // there are more than 10 sections containing text,
  // 2 = expand all
  arg_ainfo.shortname = "gt";
  arg_ainfo.longname = "expand text";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "gp" is the "go to page" control
  // used by the Book type of toc
  arg_ainfo.shortname = "gp";
  arg_ainfo.longname = "go to page";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::none;
  arg_ainfo.argdefault = "";
  arg_ainfo.savedarginfo = cgiarginfo::mustnot;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // in this action "hl" is the "highlighting on/
  // highlighting off control
  arg_ainfo.shortname = "hl";
  arg_ainfo.longname = "highlighting on/off";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "1";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);

  // "x" is 0 normally or 1 if page
  // has been "detached"
  arg_ainfo.shortname = "x";
  arg_ainfo.longname = "detached page";
  arg_ainfo.multiplechar = false;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "0";
  arg_ainfo.savedarginfo = cgiarginfo::can;
  argsinfo.addarginfo (NULL, arg_ainfo);
}

documentaction::~documentaction () {
}

bool documentaction::init (ostream &logout) {
  return action::init (logout);
}

bool documentaction::check_cgiargs (cgiargsclass &/*args*/, ostream &/*logout*/) {
  // don't want to check anything yet.
  return true;
}

void documentaction::get_cgihead_info (cgiargsclass &/*args*/, response_t &response,
				     text_t &response_data, ostream &/*logout*/) {
  response = content;
  response_data = "text/html";
}


// define all the macros which might be used by other actions
// to produce pages. 
void documentaction::define_external_macros (displayclass &disp, cgiargsclass &args, 
					     recptproto *collectproto, ostream &logout) {
  
  // define_external_macros sets the following macros:

  // _navigationbar_     this is the navigation bar containing the search button
  //                     and any classification buttons - it goes at the top of 
  //                     most pages. for now we're assuming that there'll always 
  //                     be a search button - we should probably check that there
  //                     is a query action before making this assumption

  // _httpbrowseXXX_     the http macros for each classification (i.e. if there
  //                     are Title and Creator classifications _httpbrowseTitle_
  //                     and _httpbrowseCreator_ will be set

  // _javaimagesnavbar_  this is the javascript code to shove in to make the 
  //                     flashy images used by _navigationbar_ work

  // _cgiargd_           these are overridden if they use the tricky ".xx" syntax
  // _cgiargcl_
  

  outconvertclass text_t2ascii;
  comerror_t err;
  InfoFiltersResponse_t filterinfo;
  FilterResponse_t response;
  FilterRequest_t request;
  text_tarray metadata;
  text_t navigationbar, javaimagesnavbar, width, topparent;
  int twidth, swidth, iwidth = 0;

  // if the "gp" (go to page) argument is set we need to set 
  // the "d" argument to the corresponding page
  if ((!args["d"].empty()) && (!args["gp"].empty()) &&
      (is_number (args["gp"]))) {
    text_t top;
    get_top (args["d"], top);
    metadata.push_back ("Title");
    get_children (top, args["c"], metadata, collectproto, response, logout);
    ResultDocInfo_tarray::const_iterator dochere = response.docInfo.begin();
    ResultDocInfo_tarray::const_iterator docend = response.docInfo.end();
    while (dochere != docend) {
      if ((*dochere).metadata[0].values[0] == args["gp"]) {
	args["d"] = (*dochere).OID;
	disp.setmacro ("cgiargd", "Global", args["d"]);
	break;
      }
      dochere ++;
    }
    metadata.erase (metadata.begin(), metadata.end());
  }

  // do a call to translate OIDs if required
  request.filterName = "NullFilter";
  request.filterResultOptions = FROID;
  if (needs_translating (args["d"])) {
    request.docSet.insert (args["d"]);
    collectproto->filter (args["c"], request, response, err, logout);
    args["d"] = response.docInfo[0].OID;
    request.docSet.erase (request.docSet.begin(), request.docSet.end());
    disp.setmacro ("cgiargd", "Global", args["d"]);
  }
  if (needs_translating (args["cl"])) {
    request.docSet.insert (args["cl"]);
    collectproto->filter (args["c"], request, response, err, logout);
    args["cl"] = response.docInfo[0].OID;
    disp.setmacro ("cgiargcl", "Global", args["cl"]);
  }

  get_top (args["cl"], topparent);

  collectproto->get_filterinfo (args["c"], filterinfo, err, logout);
  if (err == noError) {
    // check that there's a browse filter
    if (filterinfo.filterNames.find ("BrowseFilter") != filterinfo.filterNames.end()) {
      
      metadata.push_back ("Title");
      metadata.push_back ("classifytype");
      get_children ("", args["c"], metadata, collectproto, response, logout);

      // don't do anything unless there are classifications
      if (!response.docInfo.empty()) {

	ResultDocInfo_tarray::const_iterator dochere = response.docInfo.begin();
	ResultDocInfo_tarray::const_iterator docend = response.docInfo.end();
	
	navigationbar = "<!-- Navigation Bar -->\n";
	  
	// calculate width of spacers and set macro
	if (args.getintarg("v") == 0) {
	  disp.expandstring ("Global", "_pagewidth_", width);
	  twidth = width.getint();
	  
	  disp.expandstring ("query", "_searchwidth_", width);
	  iwidth += width.getint();
	  
	  while (dochere != docend) {
	    disp.expandstring ("document", "_" + (*dochere).metadata[0].values[0] + "width_", width);
	    iwidth += width.getint();
	    dochere ++;
	  }
	  if ((twidth - iwidth) < int(response.docInfo.size())) swidth = 2;
	  else swidth = (twidth - iwidth) / response.docInfo.size();
	  disp.setmacro ("widthtspace", "Global", swidth);
	}
	
	navigationbar += "<nobr>\n";
	if (args["a"] == "q") {
	  navigationbar += "_icontabsearchgreen_";
	} else {
	  navigationbar += "_imagesearch_";
	  javaimagesnavbar = "_javasearch_";
	}
	dochere = response.docInfo.begin();
	while (dochere != docend) {
	  const text_t &title = (*dochere).metadata[0].values[0];
	  const text_t &classifytype = (*dochere).metadata[1].values[0];

	  // if we're inside a document all the classification buttons should be enabled
	  if (args["d"].empty() && ((*dochere).OID == topparent))
	    navigationbar += "_imagespacer__icontab" + title + "green_";
	  else {
	    navigationbar += "_imagespacer__image" + title + "_";

	    // set the _httpbrowseXXX_ macro for this classification
	    text_t link = (*dochere).OID;
	    if (classifytype == "AZList" || classifytype == "Datelist") link += ".fc";
	    disp.setmacro ("httpbrowse" + title, "Global", 
			   "_httpdocument_&cl=" + link);
	  }
	  javaimagesnavbar += "_java" + title + "_";
	  dochere ++;
	}
	navigationbar += "\n</nobr>\n";
	navigationbar += "<!-- End of Navigation Bar -->\n";
	disp.setmacro ("navigationbar", "Global", navigationbar);
	if (args.getintarg("v") == 0) 
	  disp.setmacro ("javaimagesnavbar", "Global", javaimagesnavbar);
      }
    }
  } else {
    logout << text_t2ascii 
	   << "Error (documentaction::define_external_macros()) in call to get_filterinfo() " 
	   << get_comerror_string (err);
  }
}


// define all the macros which are related to pages generated
// by this action
void documentaction::define_internal_macros (displayclass &disp, cgiargsclass &args, 
					     recptproto *collectproto, ostream &logout) {

  // define_internal_macros sets the following macros:

  // _imagethispage_        the title image to be displayed at top right of page

  // _classificationlinks_  the links between classifications to be displayed
  //                        at top of list or datelist classifications

  // _httpprevarrow_        these are set if next or previous arrows
  // _httpnextarrow_        are to be used - (i.e. if it's an AZList or a Datelist 
  //                        classifytype at a top level or if it's a Book or a
  //                        Hierarchy classifytype at document level 

  // _header_               the header macro is overridden if we're not at a top level 
  //                        classification to remove the title block

  // _javaimagescontent_     this is the javascript code to shove in to make the 
  //                         flashy buttons work

  
  text_tarray metadata;
  FilterResponse_t response;
  
  metadata.push_back ("Title");
  metadata.push_back ("classifytype");
  
  if (!args["d"].empty()) {
    // we're at document level
    text_t javaimagescontent;

    // get metadata for top level of document
    text_t doctop;
    get_top (args["d"], doctop);
    if (get_info (doctop, args["c"], metadata, collectproto, response, logout)) {
      disp.setmacro ("header", "document", "_textheader_");
      text_t &classifytype = response.docInfo[0].metadata[1].values[0];
      if (classifytype.empty()) classifytype = "Book"; // defaults to Book
      if ((classifytype == "Book") || (classifytype == "Hierarchy"))
	set_arrow_macros (args["d"], classifytype, disp, collectproto, args["c"], logout);

      if (args["u"] != "1") {
	javaimagescontent += "_javadetach_";
	if (args["gt"] == "1") 
	  javaimagescontent += "_javacontracttext__javacontinue_";
	else 
	  javaimagescontent += "_javaexpandtext_";
	if (classifytype == "Hierarchy")
	  if (args["gc"] == "1")
	    javaimagescontent += "_javacontractcontents_";
	  else 
	    javaimagescontent += "_javaexpandcontents_";
	if (args["hl"] != "1")
	  javaimagescontent += "_javahighlighting_";
	else 
	  javaimagescontent += "_javanohighlighting_";
	disp.setmacro ("javaimagescontent", "document", javaimagescontent);
      }
    }
  } else {
    if (!args["cl"].empty()) {
	
      // get metadata for top level classification
      text_t classtop;
      get_top (args["cl"], classtop);
      if (get_info (classtop, args["c"], metadata, collectproto, response, logout)) {
	  
	text_t &title = response.docInfo[0].metadata[0].values[0];
	text_t &classifytype = response.docInfo[0].metadata[1].values[0];

	disp.setmacro ("imagethispage", "document", 
		       "_icon" + title + "page_");

	// now get the metadata for each child of top level
	// so we can generate the _classificationlinks_ and arrow macros
	// (if they're required by the current classification type)
	
	if ((classifytype == "AZList") || (classifytype == "Datelist")) {
	  
	  text_t classificationlinks, navarrows;
	  text_t link = "<a href=\"_httpdocument_&cl=";

	  metadata.erase(metadata.begin(), metadata.end());
	  metadata.push_back ("Title");
	  metadata.push_back ("doctype");
	  if (get_children (classtop, args["c"], metadata, collectproto, response, logout)) {
	    
	    // don't want links unless there are 2 or more sections
	    if (response.docInfo.size() >= 2) {
	      
	      classificationlinks += "<!-- Classification Links -->\n";
	      classificationlinks += "<table width=_pagewidth_ cellpadding=0 cellspacing=0 border=0>\n";
	      classificationlinks += "<tr><td valign=top><center>\n";
	      
	      ResultDocInfo_tarray::const_iterator sechere = response.docInfo.begin();
	      ResultDocInfo_tarray::const_iterator secend = response.docInfo.end();
	      
	      while (sechere != secend) {
		if ((*sechere).metadata[1].values[0] == "classify") {
		  if ((*sechere).OID == args["cl"]) {
		    
		    // set the _httpprevarrow_, _httpnextarrow_ macros while we're here
		    if (sechere != response.docInfo.begin())
		      disp.setmacro ("httpprevarrow", "document", "_httpdocument_&cl=" + (*(sechere-1)).OID);
		    if ((sechere + 1) != secend)
		      disp.setmacro ("httpnextarrow", "document", "_httpdocument_&cl=" + (*(sechere+1)).OID);

		    classificationlinks += "\n<b>" + (*sechere).metadata[0].values[0] + "</b>&nbsp;&nbsp;&nbsp;";
		  } else {
		    classificationlinks += link + (*sechere).OID + "\">";
		    classificationlinks += (*sechere).metadata[0].values[0] + "</a>";
		    if ((sechere + 1) != secend) classificationlinks += "&nbsp;&nbsp;&nbsp;";
		  }
		}
		sechere ++;
	      }
	      
	      classificationlinks += "\n</center></td></tr></table>\n";
	      classificationlinks += "<!-- End of Classification Links -->\n";
	      
	      disp.setmacro ("classificationlinks", "document", classificationlinks);
	    }
	  }
	}
      }
    }
  }
}

bool documentaction::do_action (cgiargsclass &args, recptproto *collectproto,
			      displayclass &disp, outconvertclass &outconvert, 
			      ostream &textout, ostream &logout) {

  text_tarray metadata;
  FilterResponse_t response;
  DocumentRequest_t docrequest;
  DocumentResponse_t docresponse;
  text_t topparent, classifytype, classifytitle;
  comerror_t err;

  text_t &arg_d = args["d"];
  text_t &arg_cl = args["cl"];
  text_t &collection = args["c"];

  text_t OID = arg_d;
  if (arg_d.empty()) OID = arg_cl;

  textout << outconvert << disp << "_document:header_\n"
	  << "_document:content_\n";
  
  if (arg_d.empty() && arg_cl.empty()) {
    textout << outconvert << "Document contains no data_document:footer_\n";
    return true;
  }

  // get the classifytitle and classifytype
  get_top (OID, topparent);
  metadata.push_back ("Title");
  metadata.push_back ("classifytype");
  if (get_info (topparent, collection, metadata, collectproto, response, logout)) {
    classifytitle = response.docInfo[0].metadata[0].values[0];
    classifytype = response.docInfo[0].metadata[1].values[0];
  }

  if (!arg_d.empty()) {
    
    if (classifytype == "Hierarchy")
      output_hierarchy_toc (classifytitle, args, collectproto, disp, 
			    outconvert, textout, logout);
    else
      // Book is the default for a document level toc - AZList and Datelist don't
      // make sense for document level
      output_book_toc (classifytitle, args, collectproto, disp,
		       outconvert, textout, logout);
  } else {
    
    if (classifytype == "Hierarchy")
      output_hierarchy_toc (classifytitle, args, collectproto, disp, 
			    outconvert, textout, logout);
    else if (classifytype == "Datelist")
      output_datelist_toc (classifytitle, args, collectproto, disp, 
			   outconvert, textout, logout);
    else 
      // AZList is the default for a classification level toc - Book doesn't make 
      // sense for classification level
      output_azlist_toc (classifytitle, args, collectproto, disp, 
			 outconvert, textout, logout);
  }

  // get info on this document
  metadata.erase (metadata.begin(), metadata.end());
  metadata.push_back("Title");
  metadata.push_back("hastxt");
  metadata.push_back("haschildren");

  if (get_info (OID, collection, metadata, collectproto, response, logout)) {
    text_t &title = response.docInfo[0].metadata[0].values[0];
    int hastxt = response.docInfo[0].metadata[1].values[0].getint();
    int haschildren = response.docInfo[0].metadata[2].values[0].getint();

    int gt = args.getintarg("gt");
    if (gt == 0) {
      if (hastxt == 1) {
	// get the text
	docrequest.OID = OID;
	collectproto->get_document (collection, docrequest, docresponse, err, logout);
	
	textout << outconvert << disp 
		<< "<p>\n<h3>" << title << "</h3>" << docresponse.doc;
      }
    } else {

      // text is to be expanded
      int tmp; // this doesn't get used here
      if (!haschildren)
	OID = get_parent (OID);

      get_contents (OID, classifytype, tmp, collection, collectproto, response, logout);
	      
      ResultDocInfo_tarray::const_iterator sechere = response.docInfo.begin();
      ResultDocInfo_tarray::const_iterator secend = response.docInfo.end();
	      
      if (gt == 1) {
	// check if there are more than 10 sections containing text to be expanded - 
	// if there are output warning message - this isn't a great way to do this
	// since the sections may be very large or very small - one day I'll fix it 
	// -- Stefan.
	int seccount = 0;
	while (sechere != secend) {
	  if ((*sechere).metadata[3].values[0] == "1") seccount ++;
	  if (seccount > 10) break;
	  sechere ++;
	}
	if (seccount > 10) textout << outconvert << disp << "_document:textltwarning_";
	else gt = 2;
      }

      if (gt == 2) {
	// get the text for each section
	sechere = response.docInfo.begin();
	int count = 0;
	while (sechere != secend) {
	  textout << outconvert << disp << "\n<p><a name=" << count << "><h3>" 
		  << (*sechere).metadata[0].values[0] << "</h3></a>\n";
	  if ((*sechere).metadata[3].values[0] == "1") {
	    docrequest.OID = (*sechere).OID;
	    collectproto->get_document (collection, docrequest, docresponse, err, logout);
	    textout << outconvert << disp << docresponse.doc;
	  }
	  count ++;
	  sechere ++;
	}
      }
    }
  }
  textout << outconvert << disp << "_document:footer_\n";
  return true;
}

